---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_cu.vhd]
--	Easy 1 Control Unit
--	This module describes the Easy 1 Processor 
--	Control Unit. This module receives various
--	signals used to decide the next state of the
--	processor and the signals to be sent to control
--	the components of the processor.
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee;
use ieee.std_logic_1164.all; 
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@clock  	   	- clock source
--	@reset     		- reset signal
--	@alu_op			- alu opcode
--	@pc_sel			- program counter select
--	@ir_le			- instruction register load enable 
--	@pc_is			- procgram counter is
--	@pc_le			- program counter load enable
--	@di_le			- data input register load enable
--	@ac_le			- accumulator load enable
--	@a0_sel			- address register select
--	@a0_le			- address register load enable
--	@sxt_sel		- sign extension module signal
--	@next_state		- next processor state
--	@control_bus	- control bus (Memory Operation)
--	@curr_state		- current processor state
--	@opcode			- processor opcode
--	@i				- indirect bit
--	@ac_15			- accumulator < 15 > sign bit
--

entity easy1_cu is
	
	port (
		clock 	 	: in  std_logic:='0';
		reset       : in  std_logic:='0';
		alu_op      : out std_logic_vector(2 downto 0):="000";
		pc_sel      : out std_logic_vector(1 downto 0):="00";
		ir_le       : out std_logic:='0';
		pc_is       : out std_logic:='0';
		pc_le	 	: out std_logic:='0';
		di_le       : out std_logic:='0';
		ac_le       : out std_logic:='0';
		a0_sel      : out std_logic:='0';
		a0_le       : out std_logic:='0';
		sxt_sel     : out std_logic_vector(1 downto 0):="00";
		next_state  : out std_logic_vector(3 downto 0):="0000";
		control_bus : out std_logic_vector(1 downto 0):="00";
		curr_state  : in  std_logic_vector(3 downto 0):="0000";
		opcode      : in  std_logic_vector(4 downto 0):="00000";
		i           : in  std_logic:='0';
		ac_15       : in  std_logic:='0');
	
end easy1_cu;
--
----------------------------------------------------
--	Architecture declaration
--
architecture behaviour of easy1_cu is
	
begin
	
	process(clock,curr_state,opcode)
	begin
		case clock is
			when '1' =>
			case reset is
				when '1' =>                       						-- Reset bit
				next_state <= "0000" after DELAY_1_LEVEL;           	-- Next State = Reset 1
				alu_op <= "000" after DELAY_1_LEVEL;
				pc_sel <= "00" after DELAY_1_LEVEL;
				ir_le <= '0' after DELAY_1_LEVEL;
				pc_is <= '0' after DELAY_1_LEVEL;
				pc_le <= '0' after DELAY_1_LEVEL;
				di_le <= '0' after DELAY_1_LEVEL;
				ac_le <= '0' after DELAY_1_LEVEL; 
				a0_sel <= '0' after DELAY_1_LEVEL;
				a0_le <= '0' after DELAY_1_LEVEL;
				sxt_sel <= "00" after DELAY_1_LEVEL;
				control_bus <= "00" after DELAY_1_LEVEL;
				
				when '0' =>                       						-- Reset bit
				case curr_state is
					-------------------------------------------------------------------------------
					when "0000" =>                    					-- Reset 1
					next_state <= "0001" after DELAY_1_LEVEL;          	-- Next State = Reset 2
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_sel <= "01" after DELAY_1_LEVEL;
					pc_le <= '1' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;  
					a0_le <= '0' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when "0001" =>                    					-- Reset 2
					next_state <= "0010" after DELAY_1_LEVEL;          	-- Next State = Fetch 1
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_sel <= "10" after DELAY_1_LEVEL;
					pc_is <= '1' after DELAY_1_LEVEL;
					pc_le <= '1' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_sel <= '0' after DELAY_1_LEVEL;
					a0_le <= '1' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when "0010" =>               						-- Fetch 1
					next_state <= "0011" after DELAY_1_LEVEL;      		-- Next State = Fetch 2
					control_bus <= "01" after DELAY_1_LEVEL;
					pc_le <= '0' after DELAY_1_LEVEL;
					di_le <= '1' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_le <= '0' after DELAY_1_LEVEL;
					ir_le <= '1' after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when "0011" =>               						-- Fetch 2
					case i is                  							-- Indirect bit
						when '0' =>               						-- Inmediate
						case opcode is
							when "00000" =>               				--	-- When Opcode = Compi
							next_state <= "0110" after DELAY_1_LEVEL;  	--	-- Next State = Sopr
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00001" =>               				--	-- Opcode = ShRi
							next_state <= "0110" after DELAY_1_LEVEL;   --	-- Next State = Sopr
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00010" =>               				--	-- When Opcode = Brni
							next_state <= "1100" after DELAY_1_LEVEL;	--	-- Next State = Brn 1
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00011" =>               				--	-- Opcode = Jumpi
							next_state <= "1110" after DELAY_1_LEVEL;	--	-- Next State = Jump
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00100" =>               				--	-- Opcode = Storei
							next_state <= "0111" after DELAY_1_LEVEL;   --	-- Next State = Store 1
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00101" =>               				--	-- Opcode = Loadi
							next_state <= "1001" after DELAY_1_LEVEL;   --	-- Next State = Load 1
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00110" =>               				--	-- Opcode = Andi
							next_state <= "0101" after DELAY_1_LEVEL;   --	-- Next State = Aopr
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00111" =>               				--	-- Opcode = Addi
							next_state <= "0101" after DELAY_1_LEVEL;   --	-- Next State = Aopr
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "11111" =>               				--	-- Opcode = STOP
							next_state <= "1111" after DELAY_1_LEVEL;   --	-- Next State = STOP
							control_bus <= "00" after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when others => null;
						end case;                       				-- END Inmediate
						
						when '1' =>                       				-- Indirect
						case opcode is
							when "11111" =>               				--	-- Opcode = STOP
							next_state <= "1111" after DELAY_1_LEVEL;   --	-- Next State = STOP
							control_bus <= "00" after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when others =>                				--	-- Any Indirect instruction
							next_state <= "0100" after DELAY_1_LEVEL;   --	-- Next State = Fetchop
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_sel <= '1' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
						end case;                       				-- END Indirect
						when others => null;
					end case;                           				-- END Fetch 2
					-------------------------------------------------------------------------------
					when "0100" =>                        				-- Fetchop
					case i is                          					-- Indirect bit
						when '1' =>
						case opcode is
							when "00000" =>               				--	-- Opcode = Comp
							next_state <= "0110" after DELAY_1_LEVEL;   --	-- Next State = Sopr
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00001" =>               				--	-- Opcode = ShR
							next_state <= "0110" after DELAY_1_LEVEL;	--	-- Next State = Sopr
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00010" =>               				--	-- Opcode = Brn
							next_state <= "1100" after DELAY_1_LEVEL;   --	-- Next State = Brn 1
							control_bus <= "01" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '1' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00011" =>               				--	-- Opcode = Jump
							next_state <= "1110" after DELAY_1_LEVEL;   --	-- Next State = Jump
							control_bus <= "01" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '1' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00100" =>               				--	-- Opcode = Store
							next_state <= "0111" after DELAY_1_LEVEL;   --	-- Next State = Store 1
							control_bus <= "01" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '1' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00101" =>               				--	-- Opcode = Load
							next_state <= "1001" after DELAY_1_LEVEL;   --	-- Next State = Load 1
							control_bus <= "01" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '1' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00110" =>               				--	-- Opcode = And
							next_state <= "0101" after DELAY_1_LEVEL;   --	-- Next State = Aopr
							control_bus <= "01" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '1' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when "00111" =>               				--	-- Opcode = Add
							next_state <= "0101" after DELAY_1_LEVEL;   --	-- Next State = Aopr
							control_bus <= "01" after DELAY_1_LEVEL;
							pc_le <= '0' after DELAY_1_LEVEL;
							di_le <= '1' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_le <= '0' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when others => null;
						end case;
						when others => null;
					end case;                           				-- END Fetchop
					-------------------------------------------------------------------------------
					when "0101" =>                        				-- Aopr
					case i is
						when '0' =>                       				-- Inmediate
						case opcode is
							when "00110" =>               				--	-- And
							next_state <= "0010" after DELAY_1_LEVEL;   --	-- Next State = Fetch
							alu_op <= "010" after DELAY_1_LEVEL;
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '1' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							sxt_sel <= "00" after DELAY_1_LEVEL;
							
							when "00111" =>               				--	-- Add
							next_state <= "0010" after DELAY_1_LEVEL;   --	-- Next State = Fetch
							alu_op <= "011" after DELAY_1_LEVEL;
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '1' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							sxt_sel <= "00" after DELAY_1_LEVEL;
							
							when others => null;
						end case;                       				-- END Inmediate
						
						when '1' =>                       				-- Indirect
						case opcode is
							when "00110" =>               				--	-- And
							next_state <= "0010" after DELAY_1_LEVEL;   --	-- Next State = Fetch
							alu_op <= "010" after DELAY_1_LEVEL;
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '1' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							sxt_sel <= "10" after DELAY_1_LEVEL;
							
							when "00111" =>               				--	-- Add
							next_state <= "0010" after DELAY_1_LEVEL;  	--	-- Next State = Fetch
							alu_op <= "011" after DELAY_1_LEVEL;
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '1' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							sxt_sel <= "10" after DELAY_1_LEVEL;
							
							when others => null;
						end case;                       				--	-- END Indirect
						when others => null;
					end case;                           				-- END Aopr
					-------------------------------------------------------------------------------
					when "0110" =>                        				-- Sopr
					case opcode is
						when "00000" =>                   				--	-- Comp
						next_state <= "0010" after DELAY_1_LEVEL;       --	-- Next State = Fetch
						alu_op <= "001" after DELAY_1_LEVEL;
						control_bus <= "00" after DELAY_1_LEVEL;
						pc_sel <= "10" after DELAY_1_LEVEL;
						pc_is <= '1' after DELAY_1_LEVEL;
						pc_le <= '1' after DELAY_1_LEVEL;
						di_le <= '0' after DELAY_1_LEVEL;
						ac_le <= '1' after DELAY_1_LEVEL;
						a0_sel <= '0' after DELAY_1_LEVEL;
						a0_le <= '1' after DELAY_1_LEVEL;
						ir_le <= '0' after DELAY_1_LEVEL;
						
						when "00001" =>                   				--	-- ShR
						next_state <= "0010" after DELAY_1_LEVEL;       --	-- Next State = Fetch
						alu_op <= "100" after DELAY_1_LEVEL;
						control_bus <= "00" after DELAY_1_LEVEL;
						pc_sel <= "10" after DELAY_1_LEVEL;
						pc_is <= '1' after DELAY_1_LEVEL;
						pc_le <= '1' after DELAY_1_LEVEL;
						di_le <= '0' after DELAY_1_LEVEL;
						ac_le <= '1' after DELAY_1_LEVEL;
						a0_sel <= '0' after DELAY_1_LEVEL;
						a0_le <= '1' after DELAY_1_LEVEL;
						ir_le <= '0' after DELAY_1_LEVEL;
						
						when others => null;
					end case;                           				-- END Sopr
					-------------------------------------------------------------------------------
					when "0111" =>                    					-- Store 1
					next_state <= "1000" after DELAY_1_LEVEL;           -- Next State = Store 2
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_le <= '0' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_sel <= '1' after DELAY_1_LEVEL;
					a0_le <= '1' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					sxt_sel <= "01" after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when "1000" =>                    					-- Store 2
					next_state <= "0010" after DELAY_1_LEVEL;           -- Next State = Fetch
					control_bus <= "10" after DELAY_1_LEVEL;
					pc_sel <= "10" after DELAY_1_LEVEL;
					pc_is <= '1' after DELAY_1_LEVEL;
					pc_le <= '1' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_sel <= '0' after DELAY_1_LEVEL;
					a0_le <= '1' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					
					-------------------------------------------------------------------------------
					when "1001" =>                    					-- Load 1
					next_state <= "1010" after DELAY_1_LEVEL;           -- Next State = Load 2
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_le <= '0' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_sel <= '1' after DELAY_1_LEVEL;
					a0_le <= '1' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					sxt_sel <= "01" after DELAY_1_LEVEL;
					
					-------------------------------------------------------------------------------
					when "1010" =>                    					-- Load 2
					next_state <= "1011" after DELAY_1_LEVEL;           -- Next State = Load 3
					control_bus <= "01" after DELAY_1_LEVEL;
					pc_le <= '0' after DELAY_1_LEVEL;
					di_le <= '1' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_le <= '0' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					
					-------------------------------------------------------------------------------
					when "1011" =>                    					-- Load 3
					next_state <= "0010" after DELAY_1_LEVEL;           -- Next State = Fetch
					alu_op <= "000" after DELAY_1_LEVEL;
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_sel <= "10" after DELAY_1_LEVEL;
					pc_is <= '1' after DELAY_1_LEVEL;
					pc_le <= '1' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '1' after DELAY_1_LEVEL;
					a0_sel <= '0' after DELAY_1_LEVEL;
					a0_le <= '1' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					sxt_sel <= "10" after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when "1100" =>                        				-- Brn 1
					case i is
						when '0' =>                       				--	-- Inmediate
						case ac_15 is
							when '0' =>                   				--	-- When AC < 15 > = 0
							next_state <= "0010" after DELAY_1_LEVEL;   --	-- Next State = Fetch
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when '1' =>                   				--	-- When AC < 15 > = 1
							next_state <= "1101" after DELAY_1_LEVEL;   --  -- Next State = Brn 2
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when others => null;
						end case;                       				--	-- END Inmediate
						
						when '1' =>                       				--	-- Indirect
						case ac_15 is
							when '0' =>                   				--	-- When AC < 15 > = 0
							next_state <= "0010" after DELAY_1_LEVEL;   --  -- Next State = Fetch
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when '1' =>                   				--	-- When AC < 15 > = 1
							next_state <= "1101" after DELAY_1_LEVEL;   --  -- Next State = Brn 2
							control_bus <= "00" after DELAY_1_LEVEL;
							pc_sel <= "10" after DELAY_1_LEVEL;
							pc_is <= '1' after DELAY_1_LEVEL;
							pc_le <= '1' after DELAY_1_LEVEL;
							di_le <= '0' after DELAY_1_LEVEL;
							ac_le <= '0' after DELAY_1_LEVEL;
							a0_sel <= '0' after DELAY_1_LEVEL;
							a0_le <= '1' after DELAY_1_LEVEL;
							ir_le <= '0' after DELAY_1_LEVEL;
							
							when others => null;
						end case;                       				--	-- END Indirect
						when others => null;
					end case;                           				-- END Brn 1
					-------------------------------------------------------------------------------
					when "1101" =>                        				-- Brn 2
					case i is
						when '0' =>                       				-- Inmediate
						next_state <= "0010" after DELAY_1_LEVEL;       -- Next State = Fetch
						control_bus <= "00" after DELAY_1_LEVEL;
						pc_sel <= "10" after DELAY_1_LEVEL;
						pc_is <= '0' after DELAY_1_LEVEL;
						pc_le <= '1' after DELAY_1_LEVEL;
						di_le <= '0' after DELAY_1_LEVEL;
						ac_le <= '0' after DELAY_1_LEVEL;
						a0_sel <= '1' after DELAY_1_LEVEL;
						a0_le <= '1' after DELAY_1_LEVEL;
						ir_le <= '0' after DELAY_1_LEVEL;
						sxt_sel <= "01" after DELAY_1_LEVEL;
						
						when '1' =>                       				-- Indirect
						next_state <= "0010" after DELAY_1_LEVEL;       -- Next State = Fetch
						control_bus <= "00" after DELAY_1_LEVEL;
						pc_sel <= "10" after DELAY_1_LEVEL;
						pc_is <= '0' after DELAY_1_LEVEL;
						pc_le <= '1' after DELAY_1_LEVEL;
						di_le <= '0' after DELAY_1_LEVEL;
						ac_le <= '0' after DELAY_1_LEVEL;
						a0_sel <= '1' after DELAY_1_LEVEL;
						a0_le <= '1' after DELAY_1_LEVEL;
						ir_le <= '1' after DELAY_1_LEVEL;
						sxt_sel <= "01" after DELAY_1_LEVEL;
						
						when others => null;
					end case;                           				-- END Brn 2
					-------------------------------------------------------------------------------
					when "1110" =>                    					-- Jump
					next_state <= "0010" after DELAY_1_LEVEL;           -- Next State = Fetch
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_sel <= "10" after DELAY_1_LEVEL;
					pc_is <= '0' after DELAY_1_LEVEL;
					pc_le <= '1' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					ac_le <= '0' after DELAY_1_LEVEL;
					a0_sel <= '1' after DELAY_1_LEVEL;
					a0_le <= '1' after DELAY_1_LEVEL;
					ir_le <= '1' after DELAY_1_LEVEL;
					sxt_sel <= "01" after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when "1111" =>                    					-- STOP
					next_state <= "1111" after DELAY_1_LEVEL;           -- Next State = STOP
					control_bus <= "00" after DELAY_1_LEVEL;
					pc_le <= '0' after DELAY_1_LEVEL;
					di_le <= '0' after DELAY_1_LEVEL;
					a0_le <= '0' after DELAY_1_LEVEL;
					ir_le <= '0' after DELAY_1_LEVEL;
					-------------------------------------------------------------------------------
					when others => null;
				end case;
				when others => null;
			end case;
			
			when '0' => 			 
			ac_le <= '0';		
			
			when others => null;
		end case; 
		
	end process;
end behaviour;
